# Generated by roxygen2: do not edit by hand

S3method("$",links)
S3method("$<-",links)
S3method("[",board_options)
S3method("[",link)
S3method("[",links)
S3method("[",plugins)
S3method("[",stack)
S3method("[<-",blocks)
S3method("[<-",board_options)
S3method("[<-",links)
S3method("[<-",plugins)
S3method("[<-",stack)
S3method("[<-",stacks)
S3method("[[",board_options)
S3method("[[",link)
S3method("[[",links)
S3method("[[",plugins)
S3method("[[",stack)
S3method("[[<-",blocks)
S3method("[[<-",board_options)
S3method("[[<-",links)
S3method("[[<-",plugins)
S3method("[[<-",stack)
S3method("[[<-",stacks)
S3method("names<-",blocks)
S3method("names<-",links)
S3method("names<-",plugins)
S3method("names<-",stacks)
S3method(add_block_to_stack,board)
S3method(anyDuplicated,plugins)
S3method(anyDuplicated,stack)
S3method(as.character,link)
S3method(as.character,stack)
S3method(as.data.frame,link)
S3method(as.data.frame,links)
S3method(as.list,blocks)
S3method(as.list,board_options)
S3method(as.list,link)
S3method(as.list,links)
S3method(as.list,plugin)
S3method(as.list,plugins)
S3method(as.list,stack)
S3method(as.list,stacks)
S3method(as.matrix,board)
S3method(as.matrix,links)
S3method(as_block,block)
S3method(as_blocks,block)
S3method(as_blocks,blocks)
S3method(as_blocks,board)
S3method(as_blocks,list)
S3method(as_board_option,board_option)
S3method(as_board_options,board)
S3method(as_board_options,board_option)
S3method(as_board_options,board_options)
S3method(as_board_options,list)
S3method(as_link,character)
S3method(as_link,data.frame)
S3method(as_link,link)
S3method(as_link,list)
S3method(as_links,"NULL")
S3method(as_links,board)
S3method(as_links,data.frame)
S3method(as_links,link)
S3method(as_links,links)
S3method(as_links,list)
S3method(as_plugin,list)
S3method(as_plugin,plugin)
S3method(as_plugin,plugins)
S3method(as_plugins,list)
S3method(as_plugins,plugin)
S3method(as_plugins,plugins)
S3method(as_stack,character)
S3method(as_stack,list)
S3method(as_stack,stack)
S3method(as_stacks,board)
S3method(as_stacks,character)
S3method(as_stacks,list)
S3method(as_stacks,stack)
S3method(as_stacks,stacks)
S3method(block_eval,block)
S3method(block_eval,plot_block)
S3method(block_eval_trigger,block)
S3method(block_eval_trigger,plot_block)
S3method(block_inputs,block)
S3method(block_inputs,board)
S3method(block_output,block)
S3method(block_output,data_block)
S3method(block_output,parser_block)
S3method(block_output,plot_block)
S3method(block_output,text_block)
S3method(block_output,transform_block)
S3method(block_render_trigger,block)
S3method(block_render_trigger,data_block)
S3method(block_render_trigger,parser_block)
S3method(block_render_trigger,transform_block)
S3method(block_server,block)
S3method(block_summary,block)
S3method(block_ui,block)
S3method(block_ui,board)
S3method(block_ui,data_block)
S3method(block_ui,parser_block)
S3method(block_ui,plot_block)
S3method(block_ui,text_block)
S3method(block_ui,transform_block)
S3method(blockr_app_options,board)
S3method(blockr_app_plugins,board)
S3method(blockr_app_server,board)
S3method(blockr_app_ui,board)
S3method(blockr_deser,block)
S3method(blockr_deser,blockr_ctor)
S3method(blockr_deser,blocks)
S3method(blockr_deser,board)
S3method(blockr_deser,board_option)
S3method(blockr_deser,board_options)
S3method(blockr_deser,link)
S3method(blockr_deser,links)
S3method(blockr_deser,list)
S3method(blockr_deser,stack)
S3method(blockr_deser,stacks)
S3method(blockr_ser,block)
S3method(blockr_ser,blockr_ctor)
S3method(blockr_ser,blocks)
S3method(blockr_ser,board)
S3method(blockr_ser,board_option)
S3method(blockr_ser,board_options)
S3method(blockr_ser,fixed_block)
S3method(blockr_ser,link)
S3method(blockr_ser,links)
S3method(blockr_ser,llm_model_option)
S3method(blockr_ser,stack)
S3method(blockr_ser,stacks)
S3method(blockr_test_exports,board)
S3method(board_options,block)
S3method(board_options,block_registry_entry)
S3method(board_options,board)
S3method(board_options,data_block)
S3method(board_options,parser_block)
S3method(board_options,transform_block)
S3method(board_plugins,board)
S3method(board_server,board)
S3method(board_ui,"NULL")
S3method(board_ui,board)
S3method(board_ui,board_options)
S3method(board_ui,plugin)
S3method(board_ui,plugins)
S3method(c,block)
S3method(c,blocks)
S3method(c,board_option)
S3method(c,board_options)
S3method(c,link)
S3method(c,links)
S3method(c,plugin)
S3method(c,plugins)
S3method(c,stack)
S3method(c,stacks)
S3method(duplicated,stack)
S3method(expr_server,block)
S3method(expr_ui,block)
S3method(format,block)
S3method(format,blocks)
S3method(format,board)
S3method(format,board_option)
S3method(format,board_options)
S3method(format,link)
S3method(format,links)
S3method(format,llm_model_option)
S3method(format,plugin)
S3method(format,plugins)
S3method(format,stack)
S3method(format,stacks)
S3method(insert_block_ui,board)
S3method(insert_stack_ui,board)
S3method(intersect,stack)
S3method(is.element,stack)
S3method(is_acyclic,board)
S3method(is_acyclic,links)
S3method(is_acyclic,matrix)
S3method(modify_board_links,board)
S3method(modify_board_stacks,board)
S3method(names,board_options)
S3method(names,links)
S3method(names,plugins)
S3method(print,block)
S3method(print,blocks)
S3method(print,board)
S3method(print,board_option)
S3method(print,board_options)
S3method(print,link)
S3method(print,plugin)
S3method(print,plugins)
S3method(print,stack)
S3method(print,stacks)
S3method(remove_block_from_stack,board)
S3method(remove_block_ui,board)
S3method(remove_stack_ui,board)
S3method(replay,error)
S3method(replay,message)
S3method(replay,warning)
S3method(restore_board,board)
S3method(rm_blocks,board)
S3method(serialize_board,board)
S3method(serve,block)
S3method(serve,board)
S3method(serve,character)
S3method(setdiff,stack)
S3method(setequal,stack)
S3method(show_condition,default)
S3method(show_condition,error)
S3method(show_condition,list)
S3method(show_condition,message)
S3method(show_condition,warning)
S3method(sort,board)
S3method(stack_ui,board)
S3method(stack_ui,stack)
S3method(toolbar_ui,board)
S3method(union,stack)
S3method(validate_board,board)
S3method(validate_board,default)
S3method(validate_board_option,board_name_option)
S3method(validate_board_option,dark_mode_option)
S3method(validate_board_option,default)
S3method(validate_board_option,filter_rows_option)
S3method(validate_board_option,llm_model_option)
S3method(validate_board_option,n_rows_option)
S3method(validate_board_option,page_size_option)
S3method(validate_board_option,show_conditions_option)
S3method(validate_board_option,thematic_option)
S3method(validate_stack,default)
S3method(validate_stack,stack)
S3method(vec_restore,blocks)
S3method(vec_restore,links)
S3method(vec_restore,plugins)
S3method(vec_restore,stack)
S3method(vec_restore,stacks)
export("block_name<-")
export("board_blocks<-")
export("board_links<-")
export("board_options<-")
export("board_stacks<-")
export("stack_blocks<-")
export("stack_name<-")
export(.)
export(..)
export(abort_not_null)
export(add_block_to_stack)
export(adjective_animal)
export(as_block)
export(as_blocks)
export(as_board_option)
export(as_board_options)
export(as_link)
export(as_links)
export(as_log_level)
export(as_plugin)
export(as_plugins)
export(as_stack)
export(as_stacks)
export(available_blocks)
export(available_stack_blocks)
export(bbquote)
export(block_arity)
export(block_eval)
export(block_eval_trigger)
export(block_inputs)
export(block_metadata)
export(block_name)
export(block_output)
export(block_render_trigger)
export(block_server)
export(block_summary)
export(block_ui)
export(blockr_abort)
export(blockr_app_options)
export(blockr_app_plugins)
export(blockr_app_server)
export(blockr_app_ui)
export(blockr_deser)
export(blockr_inform)
export(blockr_option)
export(blockr_ser)
export(blockr_test_exports)
export(blockr_warn)
export(blocks)
export(board_block_ids)
export(board_blocks)
export(board_ctor)
export(board_link_ids)
export(board_links)
export(board_option_category)
export(board_option_ctor)
export(board_option_default)
export(board_option_id)
export(board_option_ids)
export(board_option_server)
export(board_option_transform)
export(board_option_trigger)
export(board_option_ui)
export(board_option_value)
export(board_option_values)
export(board_options)
export(board_plugins)
export(board_server)
export(board_stack_ids)
export(board_stacks)
export(board_ui)
export(cat_logger)
export(chr_mply)
export(chr_ply)
export(chr_xtr)
export(clear_board)
export(cnd_logger)
export(coal)
export(combine_board_options)
export(create_block)
export(ctor_fun)
export(ctor_name)
export(ctor_pkg)
export(ctrl_block)
export(ctrl_block_server)
export(ctrl_block_ui)
export(custom_options)
export(custom_plugins)
export(dbl_mply)
export(dbl_ply)
export(dbl_xtr)
export(debug_log_level)
export(default_block_name)
export(default_board_options)
export(default_category)
export(default_icon)
export(default_stack_name)
export(destroy_module)
export(edit_block)
export(edit_block_server)
export(edit_block_ui)
export(edit_stack)
export(edit_stack_server)
export(edit_stack_ui)
export(error_log_level)
export(eval_env)
export(export_code)
export(export_safely)
export(expr_server)
export(expr_ui)
export(fatal_log_level)
export(filebrowser_volumes)
export(forward_ctor)
export(generate_code)
export(generate_code_server)
export(generate_code_ui)
export(generate_plugin_args)
export(get_board_option_or_default)
export(get_board_option_or_null)
export(get_board_option_value)
export(get_board_option_values)
export(get_log_level)
export(get_s3_method)
export(get_serve_obj)
export(get_session)
export(has_length)
export(id_to_sentence_case)
export(include_mermaid)
export(info_log_level)
export(insert_block_ui)
export(insert_stack_ui)
export(int_mply)
export(int_ply)
export(int_xtr)
export(is_acyclic)
export(is_block)
export(is_blockr_ctor)
export(is_blocks)
export(is_board)
export(is_board_option)
export(is_board_options)
export(is_bool)
export(is_count)
export(is_intish)
export(is_link)
export(is_links)
export(is_number)
export(is_plugin)
export(is_plugins)
export(is_scalar)
export(is_stack)
export(is_stacks)
export(is_string)
export(lgl_mply)
export(lgl_ply)
export(lgl_xtr)
export(links)
export(list_blocks)
export(log_debug)
export(log_error)
export(log_fatal)
export(log_info)
export(log_trace)
export(log_warn)
export(lst_xtr)
export(manage_blocks)
export(manage_blocks_server)
export(manage_blocks_ui)
export(manage_links)
export(manage_links_server)
export(manage_links_ui)
export(manage_stacks)
export(manage_stacks_server)
export(manage_stacks_ui)
export(map)
export(modify_board_links)
export(modify_board_stacks)
export(new_block)
export(new_board)
export(new_board_name_option)
export(new_board_option)
export(new_board_options)
export(new_csv_block)
export(new_dark_mode_option)
export(new_data_block)
export(new_dataset_block)
export(new_file_block)
export(new_filebrowser_block)
export(new_filter_rows_option)
export(new_fixed_block)
export(new_glue_block)
export(new_head_block)
export(new_link)
export(new_llm_model_option)
export(new_merge_block)
export(new_mock_session)
export(new_n_rows_option)
export(new_page_size_option)
export(new_parser_block)
export(new_plot_block)
export(new_plugin)
export(new_rbind_block)
export(new_scatter_block)
export(new_show_conditions_option)
export(new_stack)
export(new_static_block)
export(new_subset_block)
export(new_text_block)
export(new_thematic_option)
export(new_transform_block)
export(new_upload_block)
export(not_null)
export(notify)
export(notify_user)
export(notify_user_server)
export(notify_user_ui)
export(paste_enum)
export(pkg_avail)
export(pkg_file)
export(pkg_name)
export(pkg_version)
export(plugin_id)
export(plugin_server)
export(plugin_ui)
export(plugin_validator)
export(plugins)
export(preserve_board)
export(preserve_board_server)
export(preserve_board_ui)
export(rand_names)
export(register_block)
export(register_blocks)
export(registry_id_from_block)
export(registry_metadata)
export(remove_block_from_stack)
export(remove_block_ui)
export(remove_stack_ui)
export(resolve_ctor)
export(restore_board)
export(reval)
export(reval_if)
export(rm_blocks)
export(sample_letters)
export(serialize_board)
export(serve)
export(set_blockr_options)
export(set_board_option_value)
export(set_names)
export(sink_msg)
export(stack_blocks)
export(stack_name)
export(stack_ui)
export(stacks)
export(suggested_categories)
export(to_sentence_case)
export(toolbar_ui)
export(topo_sort)
export(trace_log_level)
export(unlst)
export(unregister_blocks)
export(validate_board)
export(validate_board_option)
export(validate_board_options)
export(validate_data_inputs)
export(validate_links)
export(validate_plugins)
export(validate_stack)
export(warn_log_level)
export(with_mock_context)
export(with_mock_session)
export(write_log)
import(shiny)
import(vctrs)
importFrom(cli,qty)
importFrom(evaluate,replay)
importFrom(generics,intersect)
importFrom(generics,is.element)
importFrom(generics,setdiff)
importFrom(generics,setequal)
importFrom(generics,union)
importFrom(grDevices,col2rgb)
importFrom(graphics,plot)
importFrom(methods,is)
