# opt utils

    Code
      opt
    Output
      test: test

# llm model opt

    Code
      opt1
    Output
      llm_model: Default chat function

---

    Code
      opt2
    Output
      llm_model: chat1 (chat1, chat2)

