# stacks class

    Code
      print(stk)
    Output
      <stacks[2]>
      
      a
       <stack[5]>
       Name: "stack a"
       Blocks: "a", "b", "c", "d" and "e"
      
      b
       <stack[3]>
       Name: "stack b"
       Blocks: "f", "g" and "h"

