# plugins

    Code
      print(a)
    Output
      <preserve_board<plugin>>

---

    Code
      print(cd)
    Output
      <plugins[2]>
      manage_links: <manage_links<plugin>>
      manage_stacks: <manage_stacks<plugin>>

