% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-assertions.R
\name{is_scalar}
\alias{is_scalar}
\alias{is_string}
\alias{is_bool}
\alias{is_intish}
\alias{is_count}
\alias{is_number}
\alias{not_null}
\alias{has_length}
\title{Assertions}
\usage{
is_scalar(x)

is_string(x)

is_bool(x)

is_intish(x)

is_count(x, allow_zero = TRUE)

is_number(x)

not_null(...)

has_length(x)
}
\arguments{
\item{x}{Object to check}

\item{allow_zero}{Determines whether the value 0 is considered a valid count}

\item{...}{Silently ignored}
}
\value{
Scalar logical value.
}
\description{
Utility functions, mainly intended for asserting common preconditions are
exported for convenience in dependent packages.
}
