% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/text-block.R, R/text-glue.R
\name{new_text_block}
\alias{new_text_block}
\alias{new_glue_block}
\title{Text block constructors}
\usage{
new_text_block(server, ui, class, ctor = sys.parent(), ...)

new_glue_block(text = character(), ...)
}
\arguments{
\item{server}{A function returning \code{\link[shiny:moduleServer]{shiny::moduleServer()}}}

\item{ui}{A function with a single argument (\code{ns}) returning a \code{shiny.tag}}

\item{class}{Block subclass}

\item{ctor}{String-valued constructor name or function/frame number (mostly
for internal use or when defining constructors for virtual classes)}

\item{...}{Forwarded to \code{new_text_block()} and \code{\link[=new_block]{new_block()}}}

\item{text}{String evaluated using \code{\link[glue:glue]{glue::glue()}}}
}
\value{
All blocks constructed via \code{new_text_block()} inherit from
\code{text_block}.
}
\description{
A text block produces (markdown styled) text, given some (optional) data
input.
}
\section{Glue block}{

Using \code{\link[glue:glue]{glue::glue()}}, this block allows evaluation of a text string in the
context of datasets to produce (markdown formatted) text as block result.
}

