% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-shiny.R
\name{destroy_module}
\alias{destroy_module}
\title{Destroy a Shiny module}
\usage{
destroy_module(
  id,
  what = c("inputs", "outputs", "observers"),
  session = get_session()
)
}
\arguments{
\item{id}{Module namespace id.}

\item{what}{Character vector indicating which components to destroy.
Defaults to all of \code{"inputs"}, \code{"outputs"} and \code{"observers"}.}

\item{session}{Shiny session object.}
}
\value{
The namespaced id (invisibly).
}
\description{
Cleans up inputs, outputs and/or observers associated with a Shiny module.
}
\note{
This function relies on non-documented, internal Shiny APIs that are
not officially supported or recommended to use. These internals may break
if Shiny's internal source code changes. Use with caution and only
whenever necessary.
}
\keyword{internal}
