% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-ply.R
\name{chr_ply}
\alias{chr_ply}
\alias{lgl_ply}
\alias{int_ply}
\alias{dbl_ply}
\alias{chr_mply}
\alias{lgl_mply}
\alias{int_mply}
\alias{dbl_mply}
\alias{chr_xtr}
\alias{lgl_xtr}
\alias{int_xtr}
\alias{dbl_xtr}
\alias{lst_xtr}
\alias{map}
\title{Functional programming utilities}
\usage{
chr_ply(x, fun, ..., length = 1L, use_names = FALSE)

lgl_ply(x, fun, ..., length = 1L, use_names = FALSE)

int_ply(x, fun, ..., length = 1L, use_names = FALSE)

dbl_ply(x, fun, ..., length = 1L, use_names = FALSE)

chr_mply(..., length = 1L)

lgl_mply(..., length = 1L)

int_mply(..., length = 1L)

dbl_mply(..., length = 1L)

chr_xtr(x, i, ...)

lgl_xtr(x, i, ...)

int_xtr(x, i, ...)

dbl_xtr(x, i, ...)

lst_xtr(x, ...)

map(fun, ..., use_names = FALSE)
}
\arguments{
\item{x}{Object to iterate over}

\item{fun}{Function to apply to each component}

\item{...}{Forwarded to \code{fun}}

\item{length}{Expected result length}

\item{use_names}{Name the result using \code{names(x)}}

\item{i}{Index to extract}
}
\value{
The result of a call to \code{\link[base:lapply]{base::lapply()}}, \code{\link[base:vapply]{base::vapply()}} or
\code{\link[base:Map]{base::Map()}}.
}
\description{
A set of wrappers for \code{\link[base:vapply]{base::vapply()}} with some convenient defaults.
Intended mainly for internal use, but available for dependent packages
wherever this is deemed convenient.
}
\keyword{internal}
