% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-cnd.R
\name{blockr_abort}
\alias{blockr_abort}
\alias{blockr_warn}
\alias{blockr_inform}
\title{Blockr conditions}
\usage{
blockr_abort(..., class = character(), envir = parent.frame())

blockr_warn(
  ...,
  class = character(),
  envir = parent.frame(),
  frequency = "always",
  frequency_id = NULL
)

blockr_inform(
  ...,
  class = character(),
  envir = parent.frame(),
  frequency = "always",
  frequency_id = NULL
)
}
\arguments{
\item{...}{Forwarded to \code{\link[cli:pluralize]{cli::pluralize()}}}

\item{class}{Condition class}

\item{envir}{Forwarded to \code{\link[cli:pluralize]{cli::pluralize()}}}

\item{frequency, frequency_id}{Forwarded to \code{\link[rlang:warn]{rlang::warn()}}}
}
\value{
Called for side-effect of signaling conditions.
}
\description{
Wrappers for \code{\link[rlang:abort]{rlang::abort()}}, \code{\link[rlang:warn]{rlang::warn()}} and \code{\link[rlang:inform]{rlang::inform()}}. In
addition to \code{class}, conditions inherit from "blockr_error".
}
